﻿package Plugins::LDIFManip;

use Net::LDAP::LDIF;
use Net::LDAP::Entry;
use Config::General;
use MIME::Base64;
use XML::Simple;
use strict;

my $VERSION  = 1.0;
my $plugin_name = 'LDIFManip';
my $this_pkg = (caller(0))[0];

##################################################################
#  Check if IP is in the list
##################################################################
sub is_in_list {
    my ($in, $list) = @_;
    my ($item, $ok, $this_fun);
    $this_fun = (caller(0))[3];

    $ok = 0;
    foreach $item (@{$list}) {
        chomp($item);
        if ($item eq $in or $item eq '*') {
            $ok++;
        }
    }
    AOSICFG::write_ws_log('|> '.$this_fun.': Value ' . $in . ' is in list ['.join(", ", @{$list}).']: ' . $ok) if $config::aosi_debug_level > 2;
    return $ok;
}

##################################################################
#  Extraxt realm from base DN
##################################################################
sub base2realm {
    my ($base) = @_;
    my ($temp, $realm);

    ($realm) = split(/,/, $base);
    ($temp, $realm) = split(/=/, $realm);

    return $realm;
}

##################################################################
#  Write a logrotate config file
##################################################################
sub make_logrot {
    my ($base, $rot_file, $file) = @_;
    my ($code, $msg, $realm, $this_fun, $time_diff, $out);
    my ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks);
    $this_fun = (caller(0))[3];

    $realm = base2realm($base);
    #$file  = $config::logrotate_file_prefix . "_" . $realm . $config::logrotate_file_ext;
    $out   = $rot_file;
    $code  = 0;

    ($dev, $ino, $mode, $nlink, $uid, $gid, $rdev, $size, $atime, $mtime, $ctime, $blksize, $blocks) = stat($rot_file);
    $time_diff = time - $mtime;
    AOSICFG::write_ws_log("|> $this_fun:mtime=$mtime;time-mtime=$time_diff;") if $config::aosi_debug_level > 2;

    if ((!-e $rot_file) or ($time_diff > 604800)) {    # 604800 sec = 7 days
          AOSICFG::write_ws_log("|> $this_fun: creating file $rot_file for $file.") if $config::aosi_debug_level > 2;
          open(LOGR, ">$rot_file") || do { $code = -11; $out = 'Cannot create ' . $rot_file . ' for '.$file.' (' . $! . ')' };
          if ($code == 0) {
              print LOGR $file;
              #<<<
              print LOGR ' {
     rotate 5
     missingok
     notifempty
     create 0664 aosi aosi
}';
            #>>>
              close(LOGR);
          }
    }

    return $code, $out;
}

##################################################################
#  Dump whole LDAP
##################################################################
sub dumpAll {
      my ($ldap, $base, $attrs, $file) = @_;
      my ($this_fun, $mesg, $t1, $t2, $code, $mesg2, $val, $att, $ent, @entries);
      my ($out_error, $out_result, $out_code, $ldif, @values, $realm);
      $this_fun = (caller(0))[3];

      $realm = base2realm($base);
      AOSICFG::write_ws_log("|> $this_fun(in): base=$base; realm=$realm");
      $mesg = $ldap->bind($config::ldap_user, password => AOSICFG::get_pwd($config::ldap_pwd_file), version => 3);
      if ($mesg->code != 0) {
          AOSICFG::write_ws_log("|> $this_fun: invalid admin password in " . $config::ldap_pwd_file);
          $out_result = 'Invalid admin password!';
          $out_code   = $mesg->code;
      }
      else {
          AOSICFG::write_ws_log("|> $this_fun(bind_as_admin): " . $mesg->code) if $config::aosi_debug_level > 2;
          my %options = (
              base      => $base,
              filter    => '(objectclass=*)',
              attrs      => $attrs,
              timelimit => 300
          );
          $t1 = time;
          AOSICFG::write_ws_log("|> $this_fun:time1= " . $t1) if $config::aosi_debug_level > 2;
          $mesg2 = $ldap->search(%options);
          $t2 = time;
          AOSICFG::write_ws_log("|> $this_fun:time2= " . $t2) if $config::aosi_debug_level > 2;
          AOSICFG::write_ws_log("|> $this_fun:time2-time1= " . sprintf("%d", $t2 - $t1)) if $config::aosi_debug_level > 2;
          $code = $mesg2->code;

          if ($code == 0) {
              @entries = $mesg2->entries;
              AOSICFG::write_ws_log("|> $this_fun: #entries=" . $#entries) if $config::aosi_debug_level > 2;
              #AOSICFG::write_ws_log("|> $this_fun: dumping to file=" . $config::ldif_dump_file_prefix . "_" . $realm . $config::ldif_dump_file_ext) if $config::aosi_debug_level > 2;
              AOSICFG::write_ws_log("|> $this_fun: dumping to file=" . $file) if $config::aosi_debug_level > 2;
              $ldif = Net::LDAP::LDIF->new(
                  $file, "w",
                  encode  => 'base64',
                  onerror => 'warn',
                  change  => 1
              );
              if ($ldif) {
                  if ($ldif->error()) {
                      AOSICFG::write_ws_log("|> $this_fun: LDIF error: " . $ldif->error()) if $config::aosi_debug_level > 2;
                      $out_result = 'LDIF error: ' . $ldif->error();
                      $out_code   = -24;
                  }
                  else {
                      foreach $ent (@entries) {
                          $ent->changetype('add');
                          $ldif->write_entry($ent);
                      }
                      $out_code = 0;
                  }
              }
              $ldif->done();
          }
          else {
              $out_result = $mesg2->error;
              $out_code   = $code;
          }
          $mesg = $ldap->unbind;
          my $t3 = time;
          AOSICFG::write_ws_log("|> $this_fun:time3= " . $t2) if $config::aosi_debug_level > 2;
          AOSICFG::write_ws_log("|> $this_fun:time3-time2= " . sprintf("%d", $t3 - $t2)) if $config::aosi_debug_level > 2;
      }
      AOSICFG::write_ws_log("|> $this_fun(out): out_code=$out_code;out_result=$out_result;");
      return $out_code, $out_result;
}

##################################################################
#  Fill entries
##################################################################
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
# 4 = afterAddUser
# 5 = afterDeleteUser
sub hash2ent {
      my ($hash, $type, $attribs) = @_;
      my ($this_fun, $dn, $entry_stg, $att, $val, $att_item, $valid, $logval);
      $this_fun = (caller(0))[3];

      AOSICFG::write_ws_log("|> $this_fun(in): type=$type;hash_keys=" . join(", ", keys %$hash)) if $config::aosi_debug_level > 2;
      $dn = $hash->{'dn'};
      AOSICFG::write_ws_log("|> $this_fun: dn=$dn;") if $config::aosi_debug_level > 2;
      my $entry = Net::LDAP::Entry->new();
      $entry->dn($dn);
      if ($type <= 3) {    # Modify entry
          $entry->changetype('modify');
      }
      elsif ($type == 4) {    # Add entry
          $entry->changetype('add');
          $entry->add('objectClass' => 'person');
          $entry->add('objectClass' => 'organizationalPerson');
          $entry->add('objectClass' => 'inetOrgPerson');
          $entry->add('objectClass' => 'hrEduPerson');
          $entry->add('objectClass' => 'schacContactLocation');
      }
      elsif ($type == 5) {    # Delete entry
          $entry->changetype('delete');
          $entry->delete();
      }
      foreach $att_item (@{ $hash->{'attrs'} }) {
          if (ref($att_item) eq 'HASH') {
              $att = $att_item->{'att'};
              $val = $att_item->{'val'};
			  $logval = $att eq 'userPassword'? ['***'] : $val;
              $valid = 0;
              if(is_in_list($att, $attribs)){
                  if ($type == 1) {
                      $entry->add($att => $val);
                  }
                  elsif ($type == 2) {
                      $entry->replace($att => $val);
                  }
                  elsif ($type == 3) {
                      $entry->delete($att => $val);
                  }
                  elsif ($type == 4) {
                      $entry->add($att => $val);
                  }
                  $valid = 1;
                  AOSICFG::write_ws_log("|> $this_fun: $att = ".AOSICFG::dump_array($logval)." valid=$valid;") if $config::aosi_debug_level > 2;
              }
          }
      }
      AOSICFG::write_ws_log("|> $this_fun: entry=$entry;") if $config::aosi_debug_level > 2;

      return $entry;
}

##################################################################
#  Write to file
##################################################################
sub append_to_ldif {
      my ($entries, $type, $base_dn, $attribs, $file_in) = @_;
      my ($this_fun, $dn, $ent, $path, $uid, $base);
      $this_fun = (caller(0))[3];

      AOSICFG::write_ws_log("|> $this_fun(in): type=$type;entries_keys=" . join(", ", keys %$entries)) if $config::aosi_debug_level > 2;
      ($uid, $base) = AOSICFG::get_uid_base($entries->{'dn'});
      if (cmp_dn($base, $base_dn)) {
          my $file = $file_in || $config::ldif_sync_file_prefix . "_" . base2realm($base) . $config::ldif_sync_file_ext;

          # Hacking paths for taint
          #$file =~ /\/([^;\&]+)/;
          #$file = $1;
          AOSICFG::write_ws_log("|> $this_fun: file2=$file;") if $config::aosi_debug_level > 2;
          if (!-e $file) {
              utime(time, time, $file);
          }
          my $ldif = Net::LDAP::LDIF->new($file, "a", encode => 'base64', onerror => 'warn', change => 1);
          if ($ldif) {
              if ($ldif->error()) {
                  AOSICFG::write_ws_log("|> $this_fun: LDIF error: " . $ldif->error()) if $config::aosi_debug_level > 2;
              }
              else {
                  $ent = hash2ent($entries, $type, $attribs);
                  $ldif->write_entry($ent);
                  $ldif->done();
              }
          }
          AOSICFG::write_ws_log("|> $this_fun(out): OK") if $config::aosi_debug_level > 2;
      }
      else {
          AOSICFG::write_ws_log("|> $this_fun(out): admin base dn ($base_dn) != entry base dn ($base)") if $config::aosi_debug_level > 2;
      }
}

sub cmp_dn {
    my($a, $b) = @_;
    my(@A, @B, $i, $ok, $out);

    AOSICFG::write_ws_log("|> cmp_dn: a=$a; b=$b;") if $config::aosi_debug_level > 3;
    @A = split(/[,;]\s*/, $a);
    @B = split(/[,;]\s*/, $b);

    $ok = -1;
    foreach $i(0..$#A){
        if($A[$i] eq $B[$i]){
            $ok++;
        }
    }
    AOSICFG::write_ws_log("|> cmp_dn: ok=$ok;") if $config::aosi_debug_level > 3;

    $out = $ok == $#A ? 1 : 0;

    return $out;
}



1;
